"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _stringify = _interopRequireDefault(require("@babel/runtime/core-js/json/stringify"));

var _assign = _interopRequireDefault(require("@babel/runtime/core-js/object/assign"));

var _castArray2 = _interopRequireDefault(require("lodash/castArray"));

var _get2 = _interopRequireDefault(require("lodash/get"));

var _v = _interopRequireDefault(require("uuid/v4"));

var _bluebird = _interopRequireDefault(require("bluebird"));

var _sequelize = _interopRequireDefault(require("sequelize"));

/**
 * This memory store is meant to be a stand in for the database in unit tests.
 * It uses the same interface but instead interacts to a Map to reduce IO.
 */
var _default = async options => {
  const sequelize = new _sequelize.default(options, {
    logging: false
  });
  const node = sequelize.define('node', {
    id: {
      type: _sequelize.default.STRING,
      unique: true,
      allowNull: false,
      defaultValue: _v.default,
      primaryKey: true
    },
    numbers: {
      type: _sequelize.default.STRING
    },
    name: {
      type: _sequelize.default.STRING
    },
    upperGenerateLimit: {
      type: _sequelize.default.INTEGER,
      allowNull: false
    },
    lowerGenerateLimit: {
      type: _sequelize.default.INTEGER,
      allowNull: false
    }
  }, {
    timestamps: true,
    paranoid: true
  });
  await node.sync();
  const methods = {
    getFactories() {
      return _bluebird.default.map(node.findAll({
        raw: true
      }), res => {
        const numbers = (0, _get2.default)(res, 'numbers');
        (0, _assign.default)(res, {
          numbers: JSON.parse(numbers)
        });
        return res;
      });
    },

    /**
     * @param {UUID} id This ID will be compared to node.id
     * If there is a match then the node value will be deleted.
     */
    async deleteNodeById(id) {
      await node.destroy({
        where: {
          id
        },
        limit: 1
      });
      return methods.getNodeById(id);
    },

    /**
     * @param {UUID} id This ID will be compared to node.id
     * If there is a match then the node value will be returned.
     */
    async getNodeById(id) {
      const res = await node.findOne({
        where: {
          id
        },
        raw: true,
        paranoid: false
      });
      const numbers = (0, _get2.default)(res, 'numbers');
      (0, _assign.default)(res, {
        numbers: JSON.parse(numbers)
      });
      return res;
    },

    /**
     * @param {UUID} id the node key to upsert.
     * @param {Object} value The nodes value to upsert.
     */
    async upsertById(id, val) {
      (0, _assign.default)(val, {
        numbers: (0, _stringify.default)((0, _castArray2.default)(val.numbers))
      });
      await node.upsert(val, {
        where: {
          id
        }
      });
      return methods.getNodeById(id);
    }

  };
  return methods;
};

exports.default = _default;
//# sourceMappingURL=sequelize.js.map