"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = tree;

var _assign2 = _interopRequireDefault(require("@babel/runtime/core-js/object/assign"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _objectWithoutProperties2 = _interopRequireDefault(require("@babel/runtime/helpers/objectWithoutProperties"));

var _omit2 = _interopRequireDefault(require("lodash/fp/omit"));

var _assign3 = _interopRequireDefault(require("lodash/fp/assign"));

var _compose2 = _interopRequireDefault(require("lodash/fp/compose"));

var _mapValues2 = _interopRequireDefault(require("lodash/mapValues"));

var _random2 = _interopRequireDefault(require("lodash/random"));

var _times2 = _interopRequireDefault(require("lodash/times"));

var _noop2 = _interopRequireDefault(require("lodash/noop"));

var _faker = _interopRequireDefault(require("faker"));

var _v = _interopRequireDefault(require("uuid/v4"));

var _bluebird = _interopRequireDefault(require("bluebird"));

var _treeErrors = require("./treeErrors");

function createNode(_ref = {}) {
  let {
    id = (0, _v.default)()
  } = _ref,
      defaults = (0, _objectWithoutProperties2.default)(_ref, ["id"]);
  return (0, _objectSpread2.default)({
    id,
    factories: [],
    numbers: [],
    parent: null,
    name: _faker.default.name.findName(),
    upperGenerateLimit: 15,
    lowerGenerateLimit: 1
  }, defaults);
}

const createFactory = (0, _compose2.default)((0, _assign3.default)({
  numbers: []
}), (0, _omit2.default)('factories'), createNode);

function tree(store, callback = _noop2.default) {
  const getters = {
    getById: store.getNodeById,
    getFactories: store.getFactories
  };
  const setters = {
    async generateFactory(number) {
      const factories = [];
      (0, _times2.default)(number, () => factories.push(createFactory()));
      return _bluebird.default.map(factories, factory => store.upsertById(factory.id, factory));
    },

    async generateNumber(node, number) {
      const storedNode = await store.getNodeById(node.id);

      if (number > 15) {
        return (0, _treeErrors.maxNodesExceededError)(15);
      }

      const numbers = [];
      (0, _times2.default)(number, () => numbers.push((0, _random2.default)(storedNode.lowerGenerateLimit, storedNode.upperGenerateLimit)));
      return store.upsertById(storedNode.id, (0, _objectSpread2.default)({}, storedNode, {
        numbers
      }));
    },

    async setName(node, name) {
      const storedNode = await store.getNodeById(node.id);
      return store.upsertById(storedNode.id, (0, _objectSpread2.default)({}, storedNode, {
        name
      }));
    },

    async setLowerGenerateLimit(node, lowerGenerateLimit) {
      const storedNode = await store.getNodeById(node.id);
      return store.upsertById(storedNode.id, (0, _objectSpread2.default)({}, storedNode, {
        lowerGenerateLimit
      }));
    },

    async setUpperGenerateLimit(node, upperGenerateLimit) {
      const storedNode = await store.getNodeById(node.id);
      return store.upsertById(storedNode.id, (0, _objectSpread2.default)({}, storedNode, {
        upperGenerateLimit
      }));
    },

    deleteFactory(node) {
      return store.deleteNodeById(node.id);
    }

  };
  const callbackSetters = (0, _mapValues2.default)(setters, (fn, key) => async (...args) => {
    const res = await fn(...args);
    callback(key, res, ...args);
    return res;
  });
  return (0, _assign2.default)({}, getters, callbackSetters);
}
//# sourceMappingURL=tree.js.map